@@ -1,10 +1,9 @@
-// $Id: option.cpp,v 1.77 2002/11/06 00:58:23 ericb Exp $
+// $Id: option.cpp,v 1.86 2004/03/25 13:32:28 ericb Exp $
 //
 // This software is subject to the terms of the IBM Jikes Compiler
 // License Agreement available at the following URL:
 // http://ibm.com/developerworks/opensource/jikes.
-// Copyright (C) 1996, 1998, 1999, 2000, 2001, 2002 International Business
-// Machines Corporation and others.  All Rights Reserved.
+// Copyright (C) 1996, 2004 IBM Corporation and others.  All Rights Reserved.
 // You must accept the terms of that agreement to use this software.
 //
 
@@ -13,7 +12,7 @@
 #include "error.h"
 #include "case.h"
 #include "tab.h"
-#include "code.h"
+#include "stream.h"
 
 #ifdef HAVE_JIKES_NAMESPACE
 namespace Jikes { // Open namespace Jikes block
@@ -173,9 +172,20 @@ const wchar_t* OptionError::GetErrorMessage()
     case MISSING_OPTION_ARGUMENT:
         s << '\"' << name << "\" requires an argument.";
         break;
-    case INVALID_SDK_ARGUMENT:
-        s << '\"' << name
-          << "\" only recognizes Java SDK targets of 1.1 through 1.4.";
+    case INVALID_SOURCE_ARGUMENT:
+        if (ENABLE_SOURCE_15)
+        {
+            s << "\"-source\" only recognizes Java releases 1.3 (JLS 2 "
+              << "features), 1.4 (assert statement), and 1.5 (partial "
+              << "support beta, see NEWS for supported features).";
+            break;
+        }
+        s << "\"-source\" only recognizes Java releases 1.3 (JLS 2 features) "
+          << "and 1.4 (assert statement).";
+        break;
+    case INVALID_TARGET_ARGUMENT:
+        s << "\"-target\" only recognizes Java releases 1.1, 1.2, 1.3, 1.4, "
+          << "and 1.4.2.";
         break;
     case INVALID_K_OPTION:
         s << "No argument specified for +K option. The proper form is "
@@ -252,7 +262,8 @@ void Option::SaveCurrentDirectoryOnDisk(char c)
         if (! disk_directory)
         {
             disk_directory = new char[2];
-            strcpy(disk_directory, StringConstant::U8S_DO);
+            disk_directory[0] = U_DOT;
+            disk_directory[1] = U_NULL;
         }
 
         current_directory[Case::ToAsciiLower(c)] = disk_directory;
@@ -324,6 +335,7 @@ Option::Option(ArgumentExpander& arguments,
       dump_errors(false),
       errors(true),
       pedantic(false),
+      noassert(false),
       dependence_report_name(NULL)
 {
 #ifdef WIN32_FILE_SYSTEM
@@ -339,9 +351,8 @@ Option::Option(ArgumentExpander& arguments,
                                        main_current_directory);
     if (length > directory_length)
     {
-        // FIXME: missing a delete of main_current_directory ???
-        delete [] main_current_directory;
-        main_current_directory = StringConstant::U8S_DO;
+        main_current_directory[0] = U_DOT;
+        main_current_directory[1] = U_NULL;
         main_disk = 0;
     }
     else
@@ -362,8 +373,10 @@ Option::Option(ArgumentExpander& arguments,
 
     for (int i = 1; i < arguments.argc; i++)
     {
+        bool consumed = false;
         if (arguments.argv[i][0] == '-')
         {
+            consumed = true;
             if (strcmp(arguments.argv[i], "-bootclasspath") == 0 ||
                 strcmp(arguments.argv[i], "--bootclasspath") == 0)
             {
@@ -562,7 +575,12 @@ Option::Option(ArgumentExpander& arguments,
                      strcmp(arguments.argv[i], "--nowarn") == 0 ||
                      strcmp(arguments.argv[i], "-q") == 0)
             {
-                nowarn = true;
+                tolerance = NO_WARNINGS;
+            }
+            else if (strcmp(arguments.argv[i], "-Werror") == 0)
+            {
+                tolerance = ToleranceLevel(WARNINGS_ARE_ERRORS |
+                                           CAUTIONS_ARE_ERRORS);
             }
             else if (strcmp(arguments.argv[i], "-nowrite") == 0 ||
                      strcmp(arguments.argv[i], "--nowrite") == 0)
@@ -584,20 +602,21 @@ Option::Option(ArgumentExpander& arguments,
                                         arguments.argv[i]);
                     continue;
                 }
-                // For now, this defaults to SDK1_3 if not specified.
+                // See below for setting the default.
                 i++;
-                if (strcmp(arguments.argv[i], "1.1") == 0)
-                    source = SDK1_1;
-                else if (strcmp(arguments.argv[i], "1.2") == 0)
-                    source = SDK1_2;
-                else if (strcmp(arguments.argv[i], "1.3") == 0)
+                if (! strcmp(arguments.argv[i], "1.3"))
                     source = SDK1_3;
-                else if (strcmp(arguments.argv[i], "1.4") == 0)
+                else if (! strcmp(arguments.argv[i], "1.4"))
                     source = SDK1_4;
+                else if (ENABLE_SOURCE_15 &&
+                         ! strcmp(arguments.argv[i], "1.5"))
+                {
+                    source = SDK1_5;
+                }
                 else
                 {
                     bad_options.Next() =
-                        new OptionError(OptionError::INVALID_SDK_ARGUMENT,
+                        new OptionError(OptionError::INVALID_SOURCE_ARGUMENT,
                                         "-source");
                 }
             }
@@ -626,21 +645,24 @@ Option::Option(ArgumentExpander& arguments,
                                         arguments.argv[i]);
                     continue;
                 }
-                // This defaults to the value of source if not specified, or
-                // specified to a value less than source.
+                // See below for setting the default.
                 i++;
-                if (strcmp(arguments.argv[i], "1.1") == 0)
+                if (! strcmp(arguments.argv[i], "1.1"))
                     target = SDK1_1;
-                else if (strcmp(arguments.argv[i], "1.2") == 0)
+                else if (! strcmp(arguments.argv[i], "1.2"))
                     target = SDK1_2;
-                else if (strcmp(arguments.argv[i], "1.3") == 0)
+                else if (! strcmp(arguments.argv[i], "1.3"))
                     target = SDK1_3;
-                else if (strcmp(arguments.argv[i], "1.4") == 0)
+                else if (! strcmp(arguments.argv[i], "1.4"))
                     target = SDK1_4;
+                else if (! strcmp(arguments.argv[i], "1.4.2"))
+                    target = SDK1_4_2;
+                else if (! strcmp(arguments.argv[i], "1.5"))
+                    target = SDK1_5;
                 else
                 {
                     bad_options.Next() =
-                        new OptionError(OptionError::INVALID_SDK_ARGUMENT,
+                        new OptionError(OptionError::INVALID_TARGET_ARGUMENT,
                                         "-target");
                 }
             }
@@ -663,30 +685,51 @@ Option::Option(ArgumentExpander& arguments,
                 // flag to direct output to stdout instead of stderr.
                 //
                 Coutput.StandardOutput();
+            else if (strcmp(arguments.argv[i], "-Xswitchcheck") == 0)
+            {
+                const char* image = arguments.argv[i] + 2;
+                bool success = SemanticError::ProcessWarningSwitch(image);
+                assert(success);
+            }
             else if (arguments.argv[i][1] == 'X')
             {
-                // Note that we've already consumed -Xdepend and -Xstdout
+                // Note that we've already consumed -Xdepend, -Xstdout,
+                // and -Xswitchcheck.
                 bad_options.Next() =
                     new OptionError(OptionError::UNSUPPORTED_OPTION,
                                     arguments.argv[i]);
             }
-            else
+            else if (arguments.argv[i][1] != '-')
             {
                 bad_options.Next() =
                     new OptionError(OptionError::INVALID_OPTION,
                                     arguments.argv[i]);
             }
+            else consumed = false;
         }
-        else if (arguments.argv[i][0] == '+')
+        if (! consumed &&
+            (arguments.argv[i][0] == '+' ||
+             (arguments.argv[i][0] == '-' && arguments.argv[i][1] == '-')))
         {
-            if (strcmp(arguments.argv[i], "++") == 0)
+            consumed = true;
+            if (strcmp(arguments.argv[i], "++") == 0 ||
+                strcmp(arguments.argv[i], "--incremental") == 0)
             {
                 incremental = true;
                 full_check = true;
             }
-            else if (strcmp(arguments.argv[i], "+B") == 0)
+            else if (strcmp(arguments.argv[i], "+a") == 0 ||
+                     strcmp(arguments.argv[i], "--noassert") == 0)
+            {
+                noassert = true;
+            }
+            else if (strcmp(arguments.argv[i], "+B") == 0 ||
+                     strcmp(arguments.argv[i], "--nobytecode") == 0)
+            {
                 bytecode = false;
-            else if (strcmp(arguments.argv[i], "+D") == 0)
+            }
+            else if (strcmp(arguments.argv[i], "+D") == 0 ||
+                     strcmp(arguments.argv[i], "--dump-errors") == 0)
             {
                 dump_errors = true;
                 errors = false;
@@ -714,10 +757,16 @@ Option::Option(ArgumentExpander& arguments,
                 dependence_report_name = new char[strlen(image) + 1];
                 strcpy(dependence_report_name, image);
             }
-            else if (strcmp(arguments.argv[i], "+E") == 0)
+            else if (strcmp(arguments.argv[i], "+E") == 0 ||
+                     strcmp(arguments.argv[i], "--emacs") == 0)
+            {
                 errors = false;
-            else if (strcmp(arguments.argv[i], "+F") == 0)
+            }
+            else if (strcmp(arguments.argv[i], "+F") == 0 ||
+                     strcmp(arguments.argv[i], "--full-dependence") == 0)
+            {
                 full_check = true;
+            }
             else if (arguments.argv[i][1] == 'K')
             {
                 char *name = arguments.argv[i] + 2,
@@ -769,14 +818,16 @@ Option::Option(ArgumentExpander& arguments,
                     }
                 }
             }
-            else if (strcmp(arguments.argv[i], "+M") == 0)
+            else if (strcmp(arguments.argv[i], "+M") == 0 ||
+                     strcmp(arguments.argv[i], "--makefile") == 0)
             {
                 makefile = true;
                 full_check = true;
             }
             else if (strcmp(arguments.argv[i], "+OLDCSO") == 0)
                 old_classpath_search_order = true;
-            else if (strcmp(arguments.argv[i], "+P") == 0)
+            else if (strcmp(arguments.argv[i], "+P") == 0 ||
+                     strcmp(arguments.argv[i], "--pedantic") == 0)
             {
                 // Turn on ALL default pedantic warnings. Can be called
                 // multiple times.
@@ -785,7 +836,7 @@ Option::Option(ArgumentExpander& arguments,
             }
             else if (arguments.argv[i][1] == 'P')
             {
-                const char *image = arguments.argv[i] + 2;
+                const char* image = arguments.argv[i] + 2;
                 if (! SemanticError::ProcessWarningSwitch(image))
                 {
                     bad_options.Next() =
@@ -793,10 +844,13 @@ Option::Option(ArgumentExpander& arguments,
                                         image);
                 }
             }
-            else if (arguments.argv[i][1] == 'T')
+            else if (arguments.argv[i][1] == 'T' ||
+                     strncmp(arguments.argv[i], "--tab", 5) == 0)
             {
                 int tab_size = 0;
-                char *image = arguments.argv[i] + 2;
+                char* image = arguments.argv[i] + 2;
+                if (arguments.argv[i][1] == '-')
+                    image += 3;
                 if (*image == '=')
                     image++;
                 else if (! *image)
@@ -812,7 +866,7 @@ Option::Option(ArgumentExpander& arguments,
                 }
 
                 char *p;
-                for (p = image; *p && Code::IsDigit(*p); p++)
+                for (p = image; *p && *p >= '0' && *p <= '9'; p++)
                 {
                     int digit = *p - '0';
                     tab_size = tab_size * 10 + digit;
@@ -825,13 +879,45 @@ Option::Option(ArgumentExpander& arguments,
                 Tab::SetTabSize(tab_size == 0 ? Tab::DEFAULT_TAB_SIZE
                                 : tab_size);
             }
-            else if (strcmp(arguments.argv[i], "+U") == 0)
+            else if (strcmp(arguments.argv[i], "+U") == 0 ||
+                     strcmp(arguments.argv[i], "--unzip-dependence") == 0)
             {
                 unzip = true;
                 full_check = true;
             }
-            else if (strcmp(arguments.argv[i], "+Z") == 0)
-                zero_defect = true;
+            else if (strcmp(arguments.argv[i], "--zero-cautions") == 0)
+            {
+                tolerance = CAUTIONS_ARE_ERRORS;
+            }
+            else if (strncmp(arguments.argv[i], "+Z", 2) == 0)
+            {
+                // We accept +Z, +Z0, +Z1, and +Z2, where +Z means +Z1.
+                char* start = arguments.argv[i] + 2;
+                char* end = 0;
+                int level = (*start == '\0') ? 1 : strtoul(start, &end, 10);
+                if (end && *end != 0) level = 666;
+
+                // Translate the numeric level into a ToleranceLevel.
+                if (level == 0)
+                {
+                    tolerance = NO_WARNINGS;
+                }
+                else if (level == 1)
+                {
+                    tolerance = CAUTIONS_ARE_ERRORS;
+                }
+                else if (level == 2)
+                {
+                    tolerance = ToleranceLevel(WARNINGS_ARE_ERRORS |
+                                               CAUTIONS_ARE_ERRORS);
+                }
+                else
+                {
+                    bad_options.Next() =
+                        new OptionError(OptionError::INVALID_OPTION,
+                                        arguments.argv[i]);
+                }
+            }
 #ifdef JIKES_DEBUG
             else if (strcmp(arguments.argv[i], "+A") == 0)
                 debug_dump_ast = true;
@@ -860,13 +946,35 @@ Option::Option(ArgumentExpander& arguments,
                                     arguments.argv[i]);
             }
         }
-        else filename_index.Next() = i;
+        if (! consumed)
+            filename_index.Next() = i;
     }
 
     // Specify defaults for -source and -target.
     if (source == UNKNOWN)
-        source = SDK1_3;
-    if (target < source)
+    {
+        switch (target)
+        {
+        case SDK1_1:
+        case SDK1_2:
+        case SDK1_3:
+            source = SDK1_3;
+            break;
+        case UNKNOWN:
+            target = SDK1_4_2;
+            // fallthrough
+        case SDK1_4:
+        case SDK1_4_2:
+            source = SDK1_4;
+            break;
+        case SDK1_5:
+            source = ENABLE_SOURCE_15 ? SDK1_5 : SDK1_4;
+            break;
+        default:
+            assert(false && "Unexpected target level");
+        }
+    }
+    else if (target == UNKNOWN)
         target = source;
 
     if (! bootclasspath)
